@@ -64,6 +64,7 @@ class Regression():
         self.seed=seed
         self.in_vars=init_in_vars # 初始默认输入特征
         self.observation_model=observation_model # 'all'代表全变量测试，观测所有特征的效果趋势
+        self.is_check_overfit=False
 
         if n_core is None:
             self.n_core = os.cpu_count() - 1
@@ -103,8 +104,10 @@ class Regression():
             iperf = SCORERS[self.measure](Y_valid, Y_valid_pre, **self.kw_measure_args) ## 2.评价指标
             perf_list.append(iperf)
         df_pvalues =pd.concat(pvalue_list,axis=1).mean(1)
-        perfs = np.mean(perf_list)
 
+        perfs_mean = np.mean(perf_list)
+        perfs_max = np.max(perf_list)
+        perfs = perfs_max if self.is_check_overfit else perfs_mean
 
         X = pd.concat([X_train,X_valid],axis=0)
         if X.shape[1] < 2:
@@ -115,12 +118,13 @@ class Regression():
             df_corr = X.corr()
             t = np.arange(df_corr.shape[1])
             df_corr.values[t, t] = np.nan
-            corr_max = df_corr.max().max()
+            corr_max = df_corr.applymap(abs).max().max()
 
         check_pvalue = (df_pvalues < self.max_pvalue_limit).all()
-        check_perf = perfs > current_perf
+        check_perf = (perfs > current_perf) and (perfs_mean>0.95*current_perf)
         check_vif = vif < self.max_vif_limit
         check_corr = corr_max < self.max_corr_limit
+
         return check_pvalue, check_perf, perfs, check_vif, vif, check_corr, corr_max
 
     def _add_var(self, args):
@@ -185,6 +189,8 @@ class Regression():
         in_vars = self.in_vars
         current_perf = -np.inf
         logger_ch, fh_ch = _Tool.make_logger(self.logger_info,self.logger_file)
+        attempt_over=False
+        attempt_str=''
         while (True):
             iter_idx += 1
             if iter_idx > self.iter_num:
@@ -217,8 +223,6 @@ class Regression():
             if len(in_vars) == 0:
                 if logger_ch:
                     logger_ch.info('没有变量能够进入模型，建模结束')
-                if self.observation_model=='all':
-                    self.fit_observation(iter_idx, in_vars, logger_ch)
                 break
             with Pool(self.n_core) as pool:
                 result = pool.map_async(self._rm_var, zip(in_vars, repeat((in_vars, current_perf))))
@@ -228,13 +232,18 @@ class Regression():
                 in_vars.remove(rm_var)
                 current_perf = perf
             if (add_var is None) and (rm_var is None):
+                if not attempt_over:
+                    attempt_over=True
+                    attempt_str='(overfit)'
+                    self.is_check_overfit=True
+                    continue
                 if logger_ch:
                     logger_ch.info('在此轮迭代中，在满足使用者所设置条件的前提下，已经不能通过增加或删除变量来进一步提升模型的指标，建模结束')
                 if self.observation_model=='all':
                     self.fit_observation(iter_idx, in_vars, logger_ch)
                 break
             if logger_ch:
-                logger_ch.info('此轮迭代完成，当前入模变量为：%s。 当前模型性能%s为:%f' % (in_vars, self.measure, current_perf))
+                logger_ch.info('此轮迭代完成%s，当前入模变量为：%s。 当前模型性能%s为:%f' % (attempt_str, in_vars, self.measure, current_perf))
         out_vars = self.columns[~self.columns.isin(in_vars)]
         with Pool(self.n_core) as pool:
             result = pool.map_async(self._del_reason, zip(out_vars, repeat((in_vars, current_perf))))
@@ -291,7 +300,7 @@ class Regression():
         for i in col:
             assert i in self.columns,'{} 不是有效字段'.format(i)
         data = pd.concat(self.dataset[0][:2],axis=0)
-        df_corr = data.corr()
+        df_corr = data[list(col)].corr().applymap(abs)
         t = np.arange(df_corr.shape[1])
         df_corr.values[t, t] = np.nan
         corr_max = df_corr.max().max()
